@@ -49,10 +49,8 @@ class InternPafy(BasePafy):
 
         allinfo = get_video_info(self.videoid, self.callback)
 
-        if allinfo.get("age_ver"):
-            self.age_ver = True
-
-        self.callback("Fetched video info")
+        if self.callback:
+            self.callback("Fetched video info")
 
         def _get_lst(key, default="unknown", dic=allinfo):
             """ Dict get function, returns first index. """
@@ -81,20 +79,28 @@ class InternPafy(BasePafy):
             dbg("ciphertag mismatch")
             self.ciphertag = not self.ciphertag
 
+        watch_url = g.urls['watchv'] % self.videoid
+        if self.callback:
+            self.callback("Fetching watch page")
+        watchinfo = fetch_decode(watch_url)  # unicode
+        dbg("Fetched watch page")
+        if self.callback:
+            self.callback("Fetched watch page")
+        self.age_ver = re.search(r'player-age-gate-content">', watchinfo) is not None
+
         if self.ciphertag:
             dbg("Encrypted signature detected.")
 
             if not self.age_ver:
-                smaps, js_url, mainfunc, dashurl = get_js_sm(self.videoid,
-                        self.callback)
+                smaps, js_url, mainfunc = get_js_sm(watchinfo, self.callback)
                 funcmap[js_url] = mainfunc
                 self.sm, self.asm = smaps
                 self.js_url = js_url
-                dashsig = re.search(r"/s/([\w\.]+)", dashurl).group(1)
+                dashsig = re.search(r"/s/([\w\.]+)", self._dashurl).group(1)
                 dbg("decrypting dash sig")
                 goodsig = _decodesig(dashsig, js_url, self.callback)
                 self._dashurl = re.sub(r"/s/[\w\.]+",
-                                       "/signature/%s" % goodsig, dashurl)
+                                       "/signature/%s" % goodsig, self._dashurl)
 
             else:
                 s = re.search(r"/s/([\w\.]+)", self._dashurl).group(1)
@@ -258,22 +264,19 @@ def parseqs(data):
 
 def get_video_info(video_id, callback, newurl=None):
     """ Return info for video_id.  Returns dict. """
-    url = g.urls['vidinfo'] % video_id
+    # TODO: see if there is a way to avoid retrieving the embed page
+    #       just for this, or to use it for more. This was coppied from
+    #       youtube-dl.
+    embed_webpage = fetch_decode(g.urls['embed'])
+    sts = re.search(r'sts"\s*:\s*(\d+)', embed_webpage).group(1)
+
+    url = g.urls['vidinfo'] % (video_id, video_id, sts)
     url = newurl if newurl else url
     info = fetch_decode(url)  # bytes
     info = parseqs(info)  # unicode dict
     dbg("Fetched video info%s", " (age ver)" if newurl else "")
 
-    if info['status'][0] == "fail" and info['errorcode'][0] == '150' and \
-            "confirm your age" in info['reason'][0]:
-        # Video requires age verification
-        dbg("Age verification video")
-        callback("Age verification video")
-        newurl = g.urls['age_vidinfo'] % (video_id, video_id)
-        info = get_video_info(video_id, callback, newurl)
-        info.update({"age_ver": True})
-
-    elif info['status'][0] == "fail":
+    if info['status'][0] == "fail":
         reason = info['reason'][0] or "Bad video argument"
         raise IOError("Youtube says: %s [%s]" % (reason, video_id))
 
@@ -338,10 +341,12 @@ def _decodesig(sig, js_url, callback):
     mainfunction = funcmap[js_url]
 
     # fill in function argument with signature
-    callback("Decrypting signature")
+    if callback:
+        callback("Decrypting signature")
     solved = mainfunction([sig])
     dbg("Decrypted sig = %s...", solved[:30])
-    callback("Decrypted signature")
+    if callback:
+        callback("Decrypted signature")
     return solved
 
 
@@ -366,7 +371,8 @@ def fetch_cached(url, callback, encoding=None, dbg_ref="", file_prefix=""):
     else:
         data = fetch_decode(url, "utf8")  # unicode
         dbg("Fetched %s", dbg_ref)
-        callback("Fetched %s" % dbg_ref)
+        if callback:
+            callback("Fetched %s" % dbg_ref)
 
         with open(cached_filename, "w") as f:
             f.write(data)
@@ -404,7 +410,7 @@ def prune_files(path, prefix="", age_max=3600 * 24 * 14, count_max=4):
         os.unlink(f[0])
 
 
-def get_js_sm(video_id, callback):
+def get_js_sm(watchinfo, callback):
     """ Fetch watchinfo page and extract stream map and js funcs if not known.
 
     This function is needed by videos with encrypted signatures.
@@ -414,15 +420,9 @@ def get_js_sm(video_id, callback):
     Returns streammap (list of dicts), js url (str)  and funcs (dict)
 
     """
-    watch_url = g.urls['watchv'] % video_id
-    callback("Fetching watch page")
-    watchinfo = fetch_decode(watch_url)  # unicode
-    dbg("Fetched watch page")
-    callback("Fetched watch page")
     m = re.search(g.jsplayer, watchinfo)
     myjson = json.loads(m.group(1))
     stream_info = myjson['args']
-    dash_url = stream_info['dashmpd']
     sm = _extract_smap(g.UEFSM, stream_info, False)
     asm = _extract_smap(g.AF, stream_info, False)
     js_url = myjson['assets']['js']
@@ -431,7 +431,8 @@ def get_js_sm(video_id, callback):
 
     if not mainfunc:
         dbg("Fetching javascript")
-        callback("Fetching javascript")
+        if callback:
+            callback("Fetching javascript")
         javascript = fetch_cached(js_url, callback, encoding="utf8",
                                   dbg_ref="javascript", file_prefix="js-")
         mainfunc = _get_mainfunc_from_js(javascript)
@@ -440,7 +441,7 @@ def get_js_sm(video_id, callback):
         dbg("Using functions in memory extracted from %s", js_url)
         dbg("Mem contains %s js func sets", len(funcmap))
 
-    return (sm, asm), js_url, mainfunc, dash_url
+    return (sm, asm), js_url, mainfunc
 
 
 def _make_url(raw, sig, quick=True):
